/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var Vote = {
  
  // nsISupports
  QueryInterface: function Vote_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsIWebProgressListener) ||
        iid.equals(Ci.nsISupportsWeakReference) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function Vote_observe(subject, topic, data) {
  
    // do update requested by back-end
    var self = this;  
    if (topic == "netscape-vote")
      window.setTimeout(function() { self.doUpdate(subject, data); }, 0)
      
    // pref changed
    else
      window.setTimeout(function() { self.update(); }, 0); 
  },
  
  // nsIWebProgressListener
  onStateChange: function(progress, request, flags, status) {},
  onProgressChange: function(progess, request, curself, maxself, curtotal, maxtotal) {},
  onStatusChange: function(progress, request, status, message) {},
  onSecurityChange: function(progress, request, state) {},
  onLocationChange: function vote_onLocationChange(progress, request, location) {
    var self = this;
    var URL = location.spec;
    window.setTimeout(function() { self.check(URL); }, 0);
  },
  
  get currentURL() { return window.content.document.location.href; },
   
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _voteService: null,
  get voteService() { return this._voteService; },
  set voteService(val) { this._voteService = val; },

  _branch: null,
  get branch() { return this._branch; },
  set branch(val) { this._branch = val; },

  _voteItem: null,
  get voteItem() { return this._voteItem; },
  set voteItem(val) { this._voteItem = val; },
  
  onLoad: function vote_onLoad() {
            
    // get the vote service
    this.voteService = Cc["@netscape.com/vote/service;1"].
                       getService(Ci.nsIVoteService);    
    
    // get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-vote", false);
    
    // get the pref branch and add ourself as an observer
    this.branch = Cc["@mozilla.org/preferences-service;1"].
                  getService(Ci.nsIPrefService).
                  getBranch("extensions.netscape.vote.");
    this.branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
    
    // hookup progress listener
    getBrowser().addProgressListener(this, Ci.nsIWebProgress.NOTIFY_ALL);
                
    // update the UI
    var self = this;
    window.setTimeout(function() { self.update(); }, 100);                      
  },
  
  onUnload: function vote_onUnload() {
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-vote");
    this.branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);
    
    // remove progress listener
    try {
      getBrowser().removeProgressListener(this);
    } catch (e) {}
    
    // remove variables
    this.obService = null;
    this.voteService = null;     
    this.branch = null;
    this.voteItem = null;
  },
  
  doUpdate: function vote_doUpdate(subject, data) {
    var item = subject.QueryInterface(Components.interfaces.nsIVoteItem);
    if ((data == "scan") && (item.permaLink == this.currentURL)) 
      this..scan();
    else if ((data == "update") && (item.URL == this.currentURL)) {
      this.voteItem = item;   
      this.update();
    }
  },
  
  update: function vote_update() {
  
    // hide the UI
    var deck = document.getElementById("vote_deck");
    var status = document.getElementById("vote_status");
    if (deck)
      deck.setAttribute("hidden", "true");
    if (status)
      status.setAttribute("hidden", "true");
    
    // do nothing if disabled
    if (this.voteService.disabled)
      return;
      
    // do nothing without item
    var item = this.voteItem;
    if (!item)
      return;
      
    // get the elements
    var meta = document.getElementById("vote_metadata");
    var votes = document.getElementById("vote_votes");
    var sinks = document.getElementById("vote_sinks");
    var comments = document.getElementById("vote_comments");
    var box = document.getElementById("vote_box");
        
    // show metadata if submitted
    if (item.submitted) {
      if (status) {
        status.removeAttribute("hidden");
        meta.setAttribute("value", this.getString("vote.overlay.meta.label", [item.voteCount, item.sinkCount, item.commentCount]));
      }
      if (deck) {
        deck.removeAttribute("hidden");
        box.removeAttribute("hidden");
        deck.selectedIndex = 1;       
        comments.setAttribute("tooltiptext", this.getString("vote.overlay.comments.tooltip", [item.commentCount]));
                
        // already voted
        if (item.voted) {
          votes.setAttribute("disabled", "true");
          sinks.setAttribute("disabled", "true");         
          votes.setAttribute("tooltiptext", this.getString("vote.overlay.votes.tooltip.voted", [item.voteCount]));
          sinks.setAttribute("tooltiptext", this.getString("vote.overlay.sinks.tooltip.voted", [item.sinkCount]));

        // not voted
        } else {
          votes.removeAttribute("disabled");
          sinks.removeAttribute("disabled"); 
          votes.setAttribute("tooltiptext", this.getString("vote.overlay.votes.tooltip.not", [item.voteCount]));
          sinks.setAttribute("tooltiptext", this.getString("vote.overlay.sinks.tooltip.not", [item.sinkCount]));
        }          
      }

    // show share button
    } else {
      if (status)
        status.setAttribute("hidden", "true");
      
      if (deck) {
        box.setAttribute("hidden", "true");
        deck.removeAttribute("hidden");
        deck.selectedIndex = 0;
      }
    }    
  },
  
  scan: function vote_scan() {
    this.voteService.scanDocument(window.content.document);
  },
  
  check: function vote_check(URL) {
    this.voteItem = this.voteService.checkURL(URL);
    this.update();
  },
  
  showSidebar: function vote_showVotes(type) {
 
    // do nothing if item is missing
    var item = this.voteItem;
    if (!item)
      return;   
      
    var title = this.getString("vote.overlay.sidebar." + type);
    var suffix = (type == "votes") ? "/?whovoted=sidebar" : "/?whosunk=sidebar";
    openWebPanel(title, item.permaLink + suffix);       
  },
  
  submit: function vote_submit(event, type) {
      
    // do nothing on context click
    if (event.button == 3)
      return;
    
    // submit the story or update
    if (type == "story")
      this.submitStory(event);
    else
      this.submitUpdate(event, type);    
  },
  
  submitStory: function vote_submitStory(event) {
    
    // get the url and title
    var u = window.content.location;
    var t = window.content.document.title;
    var c = '';
    
    // get the description
    if (window.content.getSelection)
      c = window.content.getSelection();
    else if (window.content.document.getSelection)
      c = window.content.document.getSelection();
    else if (window.content.document.selection)
      c = window.content.document.selection.createRange().text;
    
    // open link
    var URL = this.branch.getCharPref("endpoint.submit");
    URL = URL.replace("$1", escape(u));
    URL = URL.replace("$2", escape(t));
    URL = URL.replace("$3", escape(c)); 
    this.openLink(event, URL);
  },
  
  submitUpdate: function vote_submitVote(event, type) { 
  
    // do nothing if item is missing
    var item = this.voteItem;
    if (!item)
      return;

    // update the item premptively    
    var data = {  URL: item.URL,
                  submitted: item.submitted,
                  voted: true,
                  voteCount: item.voteCount,
                  sinkCount: item.sinkCount,
                  commentCount: item.commentCount,
                  permaLink: item.permaLink
                };    
    
    var updateType = null;
    if (type == "vote") {
      updateType = Ci.nsIVoteService.TYPE_VOTE;
      ++data.voteCount;
    } else {
      updateType = Ci.nsIVoteService.TYPE_SINK;
      ++data.sinkCount;
    }
    item.init(data.URL, data.submitted, data.voted, data.voteCount, 
              data.sinkCount, data.commentCount, data.permaLink);
    this.voteItem = item;
    
    // update the UI
    this.update();
    
    // send the update to the service
    this.voteService.submitUpdate(item, updateType); 
  },
      
  openPermaLink: function vote_openPermaLink(event) {
      
    // do nothing on context click
    if (event.button == 3)
      return;
      
    // do nothing if item is missing
    var item = this.voteItem;
    if (!item)
      return;
      
    // open the link
    this.openLink(event, item.permaLink);
  },
  
  openLink: function vote_openLink(event, URL) {
    var where = whereToOpenLink(event);
    openUILinkIn(URL, where);
  },
  
  getString: function vote_getString(name, replace) { 
  
    // get the string bundle
    var bundle = document.getElementById("vote_bundle");
    if (!bundle)
      return null;
    
    // return the string  
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  } 
};  

window.addEventListener("load", function (event) { Vote.onLoad(); }, false);
window.addEventListener("unload", function(event) { Vote.onUnload(); }, false);